--- x264.c.orig	2007-02-19 21:49:55.000000000 +0000
+++ x264.c	2007-03-13 09:21:22.000000000 +0000
@@ -142,7 +142,8 @@
         "Infile can be raw YUV 4:2:0 (in which case resolution is required),\n"
         "  or YUV4MPEG 4:2:0 (*.y4m),\n"
         "  or AVI or Avisynth if compiled with AVIS support (%s).\n"
-        "Outfile type is selected by filename:\n"
+        "Outfile type is by default selected by filename extension:\n"
+        "                          (see options below to override)\n"
         " .264 -> Raw bytestream\n"
         " .mkv -> Matroska\n"
         " .mp4 -> MP4 if compiled with GPAC support (%s)\n"
@@ -302,6 +303,28 @@
     H0( "Input/Output:\n" );
     H0( "\n" );
     H0( "  -o, --output                Specify output file\n" );
+    H0( "      --output-format <string> Force output file format to one of the following:\n");
+    H0( "                                  264 -> Raw bytestream (default format)\n");
+    H0( "                                  mkv -> Matroska\n");
+    H0( "                                  mp4 -> MP4 if compiled with GPAC support (%s)\n",
+#ifdef MP4_OUTPUT
+        "yes"
+#else
+        "no"
+#endif
+    );
+    H0( "      --input-format <string> Force input file format to one of the following:\n");
+    H0( "                                  avi or avs -> AVI or Avisynth (default format)\n");
+    H0( "                                           if compiled with AVIS support (%s)\n",
+#ifdef AVIS_INPUT
+        "yes"
+#else
+        "no"
+#endif
+    );
+    H0( "                                  y4m -> YUV4MPEG 4:2:0\n");
+    H0( "                                  raw -> raw YUV 4:2:0\n");
+    H0( "                                      (resolution must also be specified for raw)\n");
     H0( "      --sar width:height      Specify Sample Aspect Ratio\n" );
     H0( "      --fps <float|rational>  Specify framerate\n" );
     H0( "      --seek <integer>        First frame to encode\n" );
@@ -329,28 +352,15 @@
 static int  Parse( int argc, char **argv,
                    x264_param_t *param, cli_opt_t *opt )
 {
-    char *psz_filename = NULL;
     x264_param_t defaults = *param;
-    char *psz;
     int b_avis = 0;
     int b_y4m = 0;
     int b_thread_input = 0;
+    char *out_fname = NULL, *out_fmt = NULL, *out_ext;
+    char *in_fname = NULL, *in_fmt = NULL, *in_ext;
 
     memset( opt, 0, sizeof(cli_opt_t) );
 
-    /* Default input file driver */
-    p_open_infile = open_file_yuv;
-    p_get_frame_total = get_frame_total_yuv;
-    p_read_frame = read_frame_yuv;
-    p_close_infile = close_file_yuv;
-
-    /* Default output file driver */
-    p_open_outfile = open_file_bsf;
-    p_set_outfile_param = set_param_bsf;
-    p_write_nalu = write_nalu_bsf;
-    p_set_eop = set_eop_bsf;
-    p_close_outfile = close_file_bsf;
-
     /* Parse command line options */
     for( ;; )
     {
@@ -365,6 +375,8 @@
 #define OPT_PROGRESS 261
 #define OPT_VISUALIZE 262
 #define OPT_LONGHELP 263
+#define OPT_OUTFMT 264
+#define OPT_INFMT 265
 
         static struct option long_options[] =
         {
@@ -398,6 +410,8 @@
             { "frames",  required_argument, NULL, OPT_FRAMES },
             { "seek",    required_argument, NULL, OPT_SEEK },
             { "output",  required_argument, NULL, 'o' },
+            { "output-format", required_argument, NULL, OPT_OUTFMT },
+            { "input-format", required_argument, NULL, OPT_INFMT },
             { "analyse", required_argument, NULL, 0 },
             { "partitions", required_argument, NULL, 'A' },
             { "direct",  required_argument, NULL, 0 },
@@ -498,34 +512,13 @@
                 opt->i_seek = atoi( optarg );
                 break;
             case 'o':
-                if( !strncasecmp(optarg + strlen(optarg) - 4, ".mp4", 4) )
-                {
-#ifdef MP4_OUTPUT
-                    p_open_outfile = open_file_mp4;
-                    p_write_nalu = write_nalu_mp4;
-                    p_set_outfile_param = set_param_mp4;
-                    p_set_eop = set_eop_mp4;
-                    p_close_outfile = close_file_mp4;
-#else
-                    fprintf( stderr, "x264 [error]: not compiled with MP4 output support\n" );
-                    return -1;
-#endif
-                }
-                else if( !strncasecmp(optarg + strlen(optarg) - 4, ".mkv", 4) )
-                {
-                    p_open_outfile = open_file_mkv;
-                    p_write_nalu = write_nalu_mkv;
-                    p_set_outfile_param = set_param_mkv;
-                    p_set_eop = set_eop_mkv;
-                    p_close_outfile = close_file_mkv;
-                }
-                if( !strcmp(optarg, "-") )
-                    opt->hout = stdout;
-                else if( p_open_outfile( optarg, &opt->hout ) )
-                {
-                    fprintf( stderr, "x264 [error]: can't open output file `%s'\n", optarg );
-                    return -1;
-                }
+                out_fname = strdup ( optarg );
+                break;
+            case OPT_OUTFMT:
+                out_fmt = strdup ( optarg );
+                break;
+            case OPT_INFMT:
+                in_fmt = strdup ( optarg );
                 break;
             case OPT_QPFILE:
                 opt->qpfile = fopen( optarg, "r" );
@@ -587,33 +580,98 @@
         }
     }
 
-    /* Get the file name */
+    /* Determine output format and filename and then open */
+    if( !out_fname )
+    {
+        fprintf( stderr, "x264 [error]: No output file. Run x264 --help for a list of options.\n");
+        return -1;
+    }
+    out_ext=out_fname + strlen(out_fname) - 4;
+    if( out_ext < out_fname )
+        out_ext = out_fname; /* filename is < 4 chars long */
+    /* Default output file driver */
+    p_open_outfile = open_file_bsf;
+    p_set_outfile_param = set_param_bsf;
+    p_write_nalu = write_nalu_bsf;
+    p_set_eop = set_eop_bsf;
+    p_close_outfile = close_file_bsf;
+    if( (out_fmt && !strcasecmp(out_fmt, "mp4"))
+        || (!out_fmt && !strncasecmp(out_ext, ".mp4", 4)) )
+    {
+#ifdef MP4_OUTPUT
+         p_open_outfile = open_file_mp4;
+         p_write_nalu = write_nalu_mp4;
+         p_set_outfile_param = set_param_mp4;
+         p_set_eop = set_eop_mp4;
+         p_close_outfile = close_file_mp4;
+#else
+         fprintf( stderr, "x264 [error]: not compiled with MP4 output support\n" );
+         return -1;
+#endif
+    }
+    else if( (out_fmt && !strcasecmp(out_fmt, "mkv"))
+        || (!out_fmt && !strncasecmp(out_ext, ".mkv", 4)) )
+    {
+        p_open_outfile = open_file_mkv;
+        p_write_nalu = write_nalu_mkv;
+        p_set_outfile_param = set_param_mkv;
+        p_set_eop = set_eop_mkv;
+        p_close_outfile = close_file_mkv;
+    }
+    else if( out_fmt && strcmp(out_fmt, "264") )
+    {
+        fprintf( stderr, "x264 [error]: Invalid output file format.  "
+                         "Run x264 --help for a list of options.\n" );
+        return -1;
+    }
+    if( !strcmp(out_fname, "-") )
+        opt->hout = stdout;
+    else if( p_open_outfile( out_fname, &opt->hout ) )
+    {
+         fprintf( stderr, "x264 [error]: can't open output file `%s'\n", out_fname );
+         return -1;
+    }
+    if( out_fmt )
+        free( out_fmt );
+    free( out_fname );
+    /* End of output format determination and file open */
+
+    /* Determine input format and filename and then open */
+    /* Get the input file name */
     if( optind > argc - 1 || !opt->hout )
     {
         fprintf( stderr, "x264 [error]: No %s file. Run x264 --help for a list of options.\n",
                  optind > argc - 1 ? "input" : "output" );
         return -1;
     }
-    psz_filename = argv[optind++];
+    in_fname = argv[optind++];
+    in_ext=in_fname + strlen(in_fname) - 4;
+    if( in_ext < in_fname )
+        in_ext = in_fname; /* filename is < 4 chars long */
 
-    /* check demuxer type */
-    psz = psz_filename + strlen(psz_filename) - 1;
-    while( psz > psz_filename && *psz != '.' )
-        psz--;
-    if( !strncasecmp( psz, ".avi", 4 ) || !strncasecmp( psz, ".avs", 4 ) )
+    if( (in_fmt && (!strcasecmp( in_fmt, "avi" ) || !strcasecmp( in_fmt, "avs" )))
+        || (!in_fmt && (!strncasecmp( in_ext, ".avi", 4 ) || !strncasecmp( in_ext, ".avs", 4 ))) )
         b_avis = 1;
-    if( !strncasecmp( psz, ".y4m", 4 ) )
+    else if( (in_fmt && !strcasecmp( in_fmt, "y4m" ))
+        || (!in_fmt && !strncasecmp( in_ext, ".y4m", 4 )) )
         b_y4m = 1;
+    else if( in_fmt && strcasecmp( in_fmt, "raw" ) )
+    {
+        fprintf( stderr, "x264 [error]: Invalid input file format.  "
+                         "Run x264 --help for a list of options.\n" );
+        return -1;
+    }
 
     if( !(b_avis || b_y4m) ) // raw yuv
     {
         if( optind > argc - 1 )
         {
+            char *p;
             /* try to parse the file name */
-            for( psz = psz_filename; *psz; psz++ )
+            for( p = in_fname; *p; p++ )
             {
-                if( *psz >= '0' && *psz <= '9'
-                    && sscanf( psz, "%ux%u", &param->i_width, &param->i_height ) == 2 )
+                if( *p >= '0' && *p <= '9'
+                    && sscanf( p, "%ux%u", &param->i_width, &param->i_height ) == 2 )
                 {
                     if( param->i_log_level >= X264_LOG_INFO )
                         fprintf( stderr, "x264 [info]: file name gives %dx%d\n", param->i_width, param->i_height );
@@ -633,6 +691,12 @@
         return -1;
     }
 
+    /* Default input file driver */
+    p_open_infile = open_file_yuv;
+    p_get_frame_total = get_frame_total_yuv;
+    p_read_frame = read_frame_yuv;
+    p_close_infile = close_file_yuv;
+
     /* open the input */
     {
         if( b_avis )
@@ -647,7 +711,7 @@
             return -1;
 #endif
         }
-        if ( b_y4m )
+        else if ( b_y4m )
         {
             p_open_infile = open_file_y4m;
             p_get_frame_total = get_frame_total_y4m;
@@ -655,13 +719,17 @@
             p_close_infile = close_file_y4m;
         }
 
-        if( p_open_infile( psz_filename, &opt->hin, param ) )
+        if( p_open_infile( in_fname, &opt->hin, param ) )
         {
-            fprintf( stderr, "x264 [error]: could not open input file '%s'\n", psz_filename );
+            fprintf( stderr, "x264 [error]: could not open input file '%s'\n", in_fname );
             return -1;
         }
     }
 
+    if ( in_fmt )
+        free( in_fmt );
+    free( in_fname );
+
 #ifdef HAVE_PTHREAD
     if( b_thread_input || param->i_threads > 1 )
     {